"""
    :platform: Windows (IronPython)
    :synopsis: Controls the a non-stop head.

    .. moduleauthor:: RJ Bates<rj@junipersys.com>
"""
from HM.Devices.DeviceProperties import PickListValueFormatter
from System.Collections.Generic import List
from System.Threading.Tasks import Task, TaskScheduler, TaskCreationOptions
from System.Threading import CancellationToken
from JS.Common.Units import Unit

def run_as_task(action):
	return Task.Factory.StartNew(action, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default)

class NonStopHead:
	def __init__(self):
		logger.Debug("Creating NonStop plugin")
 		self.version = "1.0.1"

		self._raise_delay_name = "RaiseDelay"
		self._raised_duration_name = "RaisedDuration"
		self._head_control_actuator_name = "HeadControlActuator"
		self._clutch_control_actuator_name = "ClutchControlActuator"
		self._brake_control_actuator_name = "brakeControlActuator"
		self._brake_enable_delay_name = "brakeEnableDelay"
		self._clutch_enable_delay_name = "clutchEnableDelay"
		self._control_task = None

		#these are the default settings change these to update defaults
		self.default_raise_delay = 2000
		self.default_raised_duration = 10000
		self.default_brake_enable_delay = 3000
		self.default_clutch_enable_delay = 2000
		self.default_head_control_actuator = "None"
		self.default_clutch_control_actuator = "Act 2.4"
		self.default_brake_control_actuator = "Act 2.7"

		self.raise_delay = self.default_raise_delay
		self.raised_duration = self.default_raised_duration
		self.brake_enable_delay = self.default_brake_enable_delay
		self.clutch_enable_delay = self.default_clutch_enable_delay
		self.head_control_actuator = self.default_head_control_actuator
		self.clutch_control_actuator = self.default_clutch_control_actuator
		self.brake_control_actuator = self.default_brake_control_actuator

	def initialize(self):
		head_control_grouping = DevicePropertyGrouping("Control", "Head")
		register_setting[int]("raise_delay", self._raise_delay_name, DevicePropertyMetadata("Start Delay", "Delay time in milliseconds to wait before raising the head."), DevicePropertyValueMetadata(NumericPropertyValueValidator(1, 100000), DevicePropertyUnits(Unit.Milliseconds)), head_control_grouping)
		register_setting[int]("raised_duration", self._raised_duration_name, DevicePropertyMetadata("Head Raised Duration","The time in milliseconds to leave the head raised."), DevicePropertyValueMetadata(NumericPropertyValueValidator(1, 100000), DevicePropertyUnits(Unit.Milliseconds)), head_control_grouping)
		register_setting[str]("head_control_actuator", self._head_control_actuator_name, DevicePropertyMetadata("Head Control Actuator","The actuator that controls the head. (None = off no actuation will be performed)"), DevicePropertyValueMetadata(StringPropertyValueValidator(), ActuatorPickListFormatter(), None, DevicePropertyUnits.None), head_control_grouping)
		register_setting[str]("clutch_control_actuator", self._clutch_control_actuator_name, DevicePropertyMetadata("Clutch Control Actuator","The actuator that controls the clutch. (None = off no actuation will be performed)"), DevicePropertyValueMetadata(StringPropertyValueValidator(), ActuatorPickListFormatter(), None, DevicePropertyUnits.None), head_control_grouping)
		register_setting[str]("brake_control_actuator", self._brake_control_actuator_name, DevicePropertyMetadata("Brake Control Actuator","The actuator that controls the brake. (None = off no actuation will be performed)"), DevicePropertyValueMetadata(StringPropertyValueValidator(), ActuatorPickListFormatter(), None, DevicePropertyUnits.None), head_control_grouping)
		register_setting[int]("brake_enable_delay", self._brake_enable_delay_name, DevicePropertyMetadata("Brake Enable Delay", "Delay time in milliseconds to wait before enabling brake actuator."), DevicePropertyValueMetadata(NumericPropertyValueValidator(1, 100000), DevicePropertyUnits(Unit.Milliseconds)), head_control_grouping)
		register_setting[int]("clutch_enable_delay", self._clutch_enable_delay_name, DevicePropertyMetadata("Clutch Enable Delay", "Delay time in milliseconds to wait before enabling the clutch actuator. This is after the head has raised."), DevicePropertyValueMetadata(NumericPropertyValueValidator(1, 100000), DevicePropertyUnits(Unit.Milliseconds)), head_control_grouping)

	def attach_grain_gage(self):
		from hm800helper import validate_actuator_module_2
		validate_actuator_module_2(grain_gage)

	def detach_grain_gage(self):
		pass

	def handle_enter_press(self):
		if self._collection_enabled and self._control_task is None:
			self._control_task = run_as_task(self.control_head)

	def handle_collection_mode_changed(self, mode):
		self._collection_mode = mode

	def handle_collection_enabled_changed(self, enabled):
		self._collection_enabled = enabled
		self._collection_mode = grain_gage.CollectionMode

	def refresh_settings(self):
		update_property(self._raise_delay_name, settings.get_int('raise_delay', self.default_raise_delay))
		update_property(self._raised_duration_name, settings.get_int('raised_duration', self.default_raised_duration))
		update_property(self._head_control_actuator_name, settings.get_string('head_control_actuator', self.default_head_control_actuator))
		update_property(self._clutch_control_actuator_name, settings.get_string('clutch_control_actuator', self.default_clutch_control_actuator))
		update_property(self._brake_control_actuator_name, settings.get_string('brake_control_actuator', self.default_brake_control_actuator))
		update_property(self._brake_enable_delay_name, settings.get_int('brake_enable_delay', self.default_brake_enable_delay))
		update_property(self._clutch_enable_delay_name, settings.get_int('clutch_enable_delay', self.default_clutch_enable_delay))

	def handle_changed_setting(self, setting_name):
		if setting_name == self._raise_delay_name:
			settings.set_int('raise_delay',self.raise_delay)
		elif setting_name == self._raised_duration_name:
			settings.set_int('raised_duration',self.raised_duration)
		elif setting_name == self._head_control_actuator_name:
			settings.set_string('head_control_actuator',self.head_control_actuator)
		elif setting_name == self._clutch_control_actuator_name:
			settings.set_string('clutch_control_actuator',self.clutch_control_actuator)
		elif setting_name == self._brake_control_actuator_name:
			settings.set_string('brake_control_actuator',self.brake_control_actuator)
		elif setting_name == self._brake_enable_delay_name:
			settings.set_int('brake_enable_delay',self.brake_enable_delay)
		elif setting_name == self._clutch_enable_delay_name:
			settings.set_int('clutch_enable_delay',self.clutch_enable_delay)

	def control_head(self):
		try:
			logger.Info("Starting control_head")
			head_actuator = self.get_actuator(self.head_control_actuator)
			clutch_actuator = self.get_actuator(self.clutch_control_actuator)
			brake_actuator = self.get_actuator(self.brake_control_actuator)

			delay(self.raise_delay)
			if clutch_actuator is not None:
				logger.Info("Disengage clutch")
				actuate.open(clutch_actuator)
			delay(self.brake_enable_delay)
			if brake_actuator is not None:
				logger.Info("Enabling Brake")
				actuate.close(brake_actuator)
			if head_actuator is not None:
				logger.Info("Raising head")
				actuate.open(head_actuator)
			delay(self.raised_duration)
			if brake_actuator is not None:
				logger.Info("Disabling Brake")
				actuate.open(brake_actuator)
			delay(self.clutch_enable_delay)
			if clutch_actuator is not None:
				logger.Info("Engaging Clutch")
				actuate.close(clutch_actuator)
			if head_actuator is not None:
				logger.Info("Dropping head")
				actuate.close(head_actuator)
		except Exception as e:
			s = str(e)
			attachment.RaiseScriptError(s)
		finally:
			self._control_task = None

	def get_actuator(self, actuator):
		if actuator == "Act 2.3":
			return act2_3
		elif actuator == "Act 2.4":
			return act2_4
		elif actuator == "Act 2.7":
			return act2_7
		elif actuator == "Act 2.8":
			return act2_8
		else:
			return None

	def handle_key_press(self, key, modifier_keys):
		if key == Key.F5:
			logger.Info("Manual Key Trigger via F5")
			if self._control_task is None:
				self._control_task = run_as_task(self.control_head)

	def setup_for_collect(self):
		pass

	def post_cycle_hook(self, observer, cycle, completed_cycles, flush):
		pass

	def post_plot_hook(self, observer, cycle, completed_cycles):
		pass

	def post_script_hook(self, observer, script_type, script_status, error_code, exception):
		pass

	def pre_cycle_hook(self, observer, cycle, completed_cycles, flush):
		pass

	def pre_plot_hook(self, observer, current_cycle, cycles_completed):
		pass

	def pre_script_hook(self, observer, script_type):
		pass

class ActuatorPickListFormatter(PickListValueFormatter):
	def get_PickListValues(self):
		values = List[str]()
		values.Add("Act 2.3")
		values.Add("Act 2.4")
		values.Add("Act 2.7")
		values.Add("Act 2.8")
		values.Add("None")
		return values

	def GetValueAsInt(self, value):
		return 0

	def GetValueAsString(self, value, units):
		return str(value)

	def GetValueFromInt(self, value):
		return None

	def GetValueFromString(self, value):
		return str(value)
